/*
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <unistd.h>

#include <glib/gstdio.h>

#include <libxfce4util/libxfce4util.h>

#include "as-file-entry.h"

#define MARGIN 2

enum
{
    TARGET_URI_LIST,
    TARGET_UTF8_STRING,
    TARGET_TEXT,
    TARGET_COMPOUND_TEXT,
    TARGET_STRING,
    TARGET_TEXT_PLAIN,
    TARGET_MOZ_URL,
};

enum {
    CHANGED_SIGNAL,
    LAST_SIGNAL
};

static const GtkTargetEntry targets[] =
{
    { "text/uri-list", 0, TARGET_URI_LIST },
    { "text/x-moz-url", 0, TARGET_MOZ_URL },
    { "UTF8_STRING", 0, TARGET_UTF8_STRING },
    { "TEXT", 0, TARGET_TEXT },
    { "COMPOUND_TEXT", 0, TARGET_COMPOUND_TEXT },
    { "STRING", 0, TARGET_STRING },
    { "text/plain", 0, TARGET_TEXT_PLAIN },
};

struct _ASFileEntryPrivate
{
    GtkWidget *entry;
    GtkWidget *button;
    GtkFileChooserAction action;
    gchar *chooser_title;
};

static guint fe_signals[LAST_SIGNAL] = { 0 };

static void as_file_entry_class_init (ASFileEntryClass *klass);
static void as_file_entry_init (ASFileEntry *fe);

G_DEFINE_TYPE (ASFileEntry, as_file_entry, GTK_TYPE_HBOX);

static void
as_file_entry_class_init (ASFileEntryClass *class)
{
    g_type_class_add_private (class, sizeof (ASFileEntryPrivate));

    /**
     * ASFileEntry::changed:
     * @widget: the widget that received the signal
     */
    fe_signals[CHANGED_SIGNAL] = g_signal_new ("changed",
                                  G_TYPE_FROM_CLASS (class),
                                  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                  G_STRUCT_OFFSET (ASFileEntryClass, entry_changed),
                                  NULL,
                                  NULL,
                                  g_cclosure_marshal_VOID__VOID,
                                  G_TYPE_NONE, 0);
}

static void
as_file_entry_init (ASFileEntry *fe)
{
    GtkWidget *img;

    fe->priv = G_TYPE_INSTANCE_GET_PRIVATE (fe, AS_TYPE_FILE_ENTRY,
                                            ASFileEntryPrivate);

    fe->priv->entry = gtk_entry_new ();
    gtk_box_pack_start (GTK_BOX (fe), fe->priv->entry,
        TRUE, TRUE, MARGIN);
    gtk_widget_show (fe->priv->entry);

    fe->priv->button = gtk_button_new ();
    img = gtk_image_new_from_stock (GTK_STOCK_OPEN,
        GTK_ICON_SIZE_BUTTON);
    if (G_LIKELY (img))
        gtk_button_set_image (GTK_BUTTON (fe->priv->button), img);
    else
        gtk_button_set_label (GTK_BUTTON (fe->priv->button), _("Select..."));
    gtk_box_pack_start (GTK_BOX (fe), fe->priv->button,
        FALSE, FALSE, MARGIN);
    gtk_widget_show (fe->priv->button);

    gtk_widget_show (GTK_WIDGET (fe));
}

static void
drag_data (GtkWidget *widget, GdkDragContext *context,
           gint x, gint y, GtkSelectionData *selection_data,
           guint info, guint time)
{
    gchar *text;
    switch (info)
    {
        case TARGET_STRING:
        case TARGET_UTF8_STRING:
        case TARGET_COMPOUND_TEXT:
        case TARGET_TEXT:
            text = (gchar *) gtk_selection_data_get_text (selection_data);
            if (G_LIKELY (text != NULL))
            {
                if (G_LIKELY (*text != '\0'))
                    gtk_entry_set_text (GTK_ENTRY (widget), text);
                g_free (text);
            }
            break;

        case TARGET_TEXT_PLAIN:
            gtk_entry_set_text (GTK_ENTRY (widget),
                (const gchar *) selection_data->data);
            break;

        case TARGET_URI_LIST:
            text = g_strndup ((const gchar *) selection_data->data,
                selection_data->length);
            gchar **uris = g_uri_list_extract_uris (text);
            g_free (text);
            /* we use the first element only */
            gchar *filename = g_filename_from_uri (uris[0], NULL, NULL);
            gtk_entry_set_text (GTK_ENTRY (widget), filename);
            g_strfreev (uris);
        break;
    }
}

static void
entry_changed (GtkWidget *w, gpointer data)
{
    g_signal_emit (G_OBJECT (data),
                   fe_signals [CHANGED_SIGNAL], 0);
}

static void
run_chooser(GtkWidget *w, gpointer data)
{
    GtkWidget *chooser;
    ASFileEntry *fe = (ASFileEntry *) data;
    gchar *filename;

    chooser = gtk_file_chooser_dialog_new (fe->priv->chooser_title,
                                           NULL,
                                           fe->priv->action,
                                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                           GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                           NULL);
    
    filename = (gchar *) gtk_entry_get_text (GTK_ENTRY (fe->priv->entry));
    if (g_access (filename, R_OK) == 0)
        gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (chooser),
            g_filename_to_uri (filename, NULL, NULL));
    else
        gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
            (gchar *) g_get_home_dir ());
    if (gtk_dialog_run (GTK_DIALOG (chooser)) == GTK_RESPONSE_ACCEPT)
    {
        gchar *filename;
        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
        gtk_entry_set_text (GTK_ENTRY (fe->priv->entry), filename);
        g_free (filename);
    }
    gtk_widget_destroy (chooser);
}

/**
 * SECTION:as-file-entry
 * @short_description: the File Entry widget
 * @include: libappstarter/as.h
 *
 */

/**
 * as_file_entry_new:
 * @title: title of the file selector
 * @action: the action that the file selector is performing
 * @file: the file name to set in the entry
 *
 * Creates a new ASFileEntry #GtkWidget.
 *
 * Returns: a GtkWidget
 */
GtkWidget *
as_file_entry_new (const gchar *title,
                   GtkFileChooserAction action,
                   const gchar *file)
{
    ASFileEntry *fe;
    fe = g_object_new (as_file_entry_get_type (), NULL);

    /* FIXME: use properties */
    fe->priv->action = action;
    fe->priv->chooser_title = g_strdup (title);

    if (file)
        gtk_entry_set_text (GTK_ENTRY (fe->priv->entry), file);
    gtk_drag_dest_set (fe->priv->entry, GTK_DEST_DEFAULT_ALL,
                       targets, G_N_ELEMENTS (targets),
                       GDK_ACTION_COPY | GDK_ACTION_LINK);

    g_signal_connect (G_OBJECT (fe->priv->entry), "drag_data_received",
        GTK_SIGNAL_FUNC (drag_data), NULL);
    g_signal_connect (G_OBJECT (fe->priv->entry), "changed",
        G_CALLBACK (entry_changed), (gpointer) fe);
    g_signal_connect (G_OBJECT (fe->priv->button), "clicked",
        G_CALLBACK (run_chooser), (gpointer) fe);
    
    return GTK_WIDGET (fe);
}

/**
 * as_file_entry_get_filename:
 * @fe: an ASFileEntry
 *
 * Get the current file name.
 *
 * Returns: the text of the entry
 */
const gchar *
as_file_entry_get_filename (ASFileEntry *fe)
{
    return gtk_entry_get_text (GTK_ENTRY (fe->priv->entry));
}

/**
 * as_file_entry_set_filename:
 * @fe: an ASFileEntry
 * @text: the text to set
 *
 * Set @text in the @fe entry.
 */
void
as_file_entry_set_filename (ASFileEntry *fe,
                            const gchar *text)
{
    gtk_entry_set_text (GTK_ENTRY (fe->priv->entry), text);
}

/**
 * as_file_entry_get_action:
 * @fe: an ASFileEntry
 *
 * Gets the type of operation that the file chooser is performing.
 *
 * Returns: the action that the file selector is performing
 */
GtkFileChooserAction
as_file_entry_get_action (ASFileEntry *fe)
{
    return fe->priv->action;
}

/**
 * as_file_entry_set_action:
 * @fe: an ASFileEntry
 * @action: the action that the file selector is performing
 *
 * Sets the type of operation that the chooser is performing.
 */
void
as_file_entry_set_action (ASFileEntry *fe,
                          GtkFileChooserAction action)
{
    fe->priv->action = action;
}

/**
 * as_file_entry_get_title:
 * @fe: an ASFileEntry
 *
 * Gets the title of the file chooser.
 *
 * Returns: the title of the file chooser
 */
const gchar *
as_file_entry_get_title (ASFileEntry *fe)
{
    return (const char *)fe->priv->chooser_title;
}

/**
 * as_file_entry_set_title:
 * @fe: an ASFileEntry
 * @title: title of the file chooser
 *
 * Sets the title of the file chooser.
 */
void
as_file_entry_set_title (ASFileEntry *fe,
                         const gchar *title)
{
    fe->priv->chooser_title = (gchar *) title;
}
